#!/bin/bash -e

unset KERNEL_REL
unset STABLE_PATCH
unset RC_PATCH
unset PRE_RC
unset BUILD
unset CC
unset LINUX_GIT
unset BISECT
unset IS_LUCID

unset LOCAL_PATCH_DIR

unset PACKAGE
unset APT

if [ ! $(which mkimage) ];then
 echo "Missing uboot-mkimage"
 PACKAGE="uboot-mkimage "
 APT=1
fi

if [ ! $(which ccache) ];then
 echo "Missing ccache"
 PACKAGE+="ccache "
 APT=1
fi

if [ ! $(which git) ];then
 echo "Missing git"
 PACKAGE+="git-core "
 APT=1
fi

if [ ! $(file /usr/lib/libncurses.so | grep -v ERROR | awk '{print $1}') ];then
 echo "Missing ncurses"
 PACKAGE+="libncurses5-dev "
 APT=1
fi

if [ "${APT}" ];then
 echo "Installing Dependicies"
 sudo aptitude install $PACKAGE
fi

DIR=$PWD

mkdir -p ${DIR}/deploy/

function git_kernel {

        cd ${LINUX_GIT}/
        git fetch
        cd -

        if [[ ! -a ${DIR}/KERNEL ]]; then
                git clone --shared ${LINUX_GIT} ${DIR}/KERNEL
        fi

        cd ${DIR}/KERNEL

        git reset --hard
        git fetch
        git checkout master

        if [ "${PRE_RC}" ]; then
                wget -c --directory-prefix=${DIR}/patches/ http://www.kernel.org/pub/linux/kernel/v2.6/snapshots/patch-${PRE_RC}.bz2
                git branch -D v${PRE_RC}-${BUILD} || true
                git checkout v${KERNEL_REL} -b v${PRE_RC}-${BUILD}
        else if [ "${RC_PATCH}" ]; then
                git branch -D v${RC_KERNEL}${RC_PATCH}-${BUILD} || true
                git checkout v${RC_KERNEL}${RC_PATCH} -b v${RC_KERNEL}${RC_PATCH}-${BUILD}
        else if [ "${STABLE_PATCH}" ] ; then
                git branch -D v${KERNEL_REL}.${STABLE_PATCH}-${BUILD} || true
                git checkout v${KERNEL_REL}.${STABLE_PATCH} -b v${KERNEL_REL}.${STABLE_PATCH}-${BUILD}
        else
                git branch -D v${KERNEL_REL}-${BUILD} || true
                git checkout v${KERNEL_REL} -b v${KERNEL_REL}-${BUILD}
        fi
        fi
        fi

        cd ${DIR}/
}

function git_bisect {
        cd ${DIR}/KERNEL

        git bisect start
        git bisect bad v2.6.35-rc2
        git bisect good v2.6.35-rc1
#        git bisect good <>

read -p "bisect look good... (y/n)? "
[ "$REPLY" == "y" ] || exit

        cd ${DIR}/
}

function patch_kernel {
        cd ${DIR}/KERNEL
        if [ "${PRE_RC}" ]; then
                bzip2 -dc ${DIR}/patches/patch-${PRE_RC}.bz2 | patch -p1 -s
        fi
        export DIR BISECT
        /bin/bash -e ${DIR}/patch.sh || { git add . ; exit 1 ; }
#exit
        git add .
        if [ "${PRE_RC}" ]; then
                git commit -a -m ''$PRE_RC'-'$BUILD' patchset'
        else if [ "${RC_PATCH}" ]; then
                git commit -a -m ''$RC_KERNEL''$RC_PATCH'-'$BUILD' patchset'
        else if [ "${STABLE_PATCH}" ] ; then
                git commit -a -m ''$KERNEL_REL''$STABLE_PATCH'-'$BUILD' patchset'
        else
                git commit -a -m ''$KERNEL_REL'-'$BUILD' patchset'
        fi
        fi
        fi
#exit
        if [ "${LOCAL_PATCH_DIR}" ]; then
                for i in ${LOCAL_PATCH_DIR}/*.patch ; do patch  -s -p1 < $i ; done
                BUILD+='+'
        fi
#exit
        cd ${DIR}/
}

function copy_defconfig {
	cd ${DIR}/KERNEL/
	make ARCH=arm CROSS_COMPILE=${CC} distclean
if [ "${IS_LUCID}" ] ; then
	cp ${DIR}/patches/lucid-defconfig .config
else
	cp ${DIR}/patches/defconfig .config
fi
	cd ${DIR}/
}

function make_menuconfig {
	cd ${DIR}/KERNEL/
	make ARCH=arm CROSS_COMPILE=${CC} menuconfig
if [ "${IS_LUCID}" ] ; then
	cp .config ${DIR}/patches/lucid-defconfig
else
	cp .config ${DIR}/patches/defconfig
fi
	cd ${DIR}/
}

function make_deb {
	cd ${DIR}/KERNEL/
	echo "make ARCH=arm LOCALVERSION=-${BUILD} CROSS_COMPILE=${CC} KDEB_PKGVERSION=${BUILDREV}${DISTRO} deb-pkg"
	fakeroot make ARCH=arm LOCALVERSION=-${BUILD} CROSS_COMPILE=${CC} KDEB_PKGVERSION=${BUILDREV}${DISTRO} deb-pkg
	mv ${DIR}/*.deb ${DIR}/deploy/
	cd ${DIR}
}

if [ -e ${DIR}/system.sh ]; then
	. system.sh
	. version.sh
if [ "-${LINUX_GIT}-" != "--" ]; then

if [ "${IS_LUCID}" ] ; then
	echo ""
	echo "Building for Lucid (10.04) & Maverick (10.10)"
	echo ""
else
	echo ""
	echo "Building for Debian Lenny/Squeeze/Sid & Ubuntu 9.04/9.10"
	echo ""
fi

	git_kernel
	#git_bisect
	patch_kernel
	copy_defconfig
	#make_menuconfig
	make_deb
else
	echo "The LINUX_GIT variable is not definted in system.sh"
	echo "Follow the git clone directions in system.sh.sample"
	echo "and make sure to remove the comment # from LINUX_GIT"
	echo "gedit system.sh"
fi
else
	echo "Missing system.sh, please copy system.sh.sample to system.sh and edit as needed"
	echo "cp system.sh.sample system.sh"
	echo "gedit system.sh"
fi

